@@ -12,19 +12,24 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import annotations
-from typing import TYPE_CHECKING
+
+from typing import TYPE_CHECKING, Any
 
 if TYPE_CHECKING:
     from supertokens_python.recipe.emailpassword.interfaces import APIOptions, APIInterface
-from .utils import validate_form_fields_or_throw_error
+
 from supertokens_python.exceptions import raise_bad_input_exception
 
+from .utils import validate_form_fields_or_throw_error
+
 
 async def handle_password_reset_api(api_implementation: APIInterface, api_options: APIOptions):
     if api_implementation.disable_generate_password_reset_token_post:
         return None
     body = await api_options.request.json()
-    form_fields_raw = body['formFields'] if 'formFields' in body else []
+    if body is None:
+        raise_bad_input_exception('Please provide a JSON body')
+    form_fields_raw: Any = body['formFields'] if 'formFields' in body else []
     form_fields = await validate_form_fields_or_throw_error(api_options.config.reset_password_using_token_feature.form_fields_for_password_reset_form,
                                                             form_fields_raw)
 
@@ -34,7 +39,7 @@ async def handle_password_reset_api(api_implementation: APIInterface, api_option
         raise_bad_input_exception('The password reset token must be a string')
 
     token = body['token']
-    response = await api_implementation.password_reset_post(form_fields, token, api_options)
+    response = await api_implementation.password_reset_post(form_fields, token, api_options, {})
     api_options.response.set_json_content(response.to_json())
 
     return api_options.response
