@@ -12,7 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from supertokens_python.exceptions import raise_bad_input_exception
-from supertokens_python.recipe.passwordless.interfaces import APIInterface, APIOptions
+from supertokens_python.recipe.passwordless.interfaces import (APIInterface,
+                                                               APIOptions)
 
 
 async def email_exists(api_implementation: APIInterface, api_options: APIOptions):
@@ -20,10 +21,10 @@ async def email_exists(api_implementation: APIInterface, api_options: APIOptions
         return None
 
     email = api_options.request.get_query_param('email')
-    if email is None or not isinstance(email, str):
+    if email is None:
         raise_bad_input_exception('Please provide the email as a GET param')
 
-    result = await api_implementation.email_exists_get(email, api_options)
+    result = await api_implementation.email_exists_get(email, api_options, {})
     api_options.response.set_json_content(result.to_json())
 
     return api_options.response
