@@ -13,28 +13,33 @@
 # under the License.
 from __future__ import annotations
 
-from typing import List, Callable, TYPE_CHECKING, Union
+from typing import (TYPE_CHECKING, Any, Awaitable, Callable, Dict, List, Set,
+                    Union)
 
-from .interfaces import RecipeInterface, APIInterface
 from supertokens_python.exceptions import raise_bad_input_exception
 
+from .interfaces import APIInterface, RecipeInterface
+
 if TYPE_CHECKING:
     from .recipe import ThirdPartyRecipe
     from .provider import Provider
-from supertokens_python.recipe.emailverification.utils import (
-    InputEmailVerificationConfig, ParentRecipeEmailVerificationConfig,
-    OverrideConfig as EmailVerificationOverrideConfig
-)
+
 from jwt import PyJWKClient, decode
+from supertokens_python.recipe.emailverification.utils import \
+    OverrideConfig as EmailVerificationOverrideConfig
+from supertokens_python.recipe.emailverification.utils import \
+    ParentRecipeEmailVerificationConfig
+
+from ..emailverification.types import User as EmailVerificationUser
+from .types import User
 
 
 class SignInAndUpFeature:
     def __init__(self, providers: List[Provider]):
         if len(providers) == 0:
-            raise_bad_input_exception('thirdparty recipe requires atleast 1 provider to be passed in '
-                                      'sign_in_and_up_feature.providers config')
-        default_providers_set = set()
-        all_providers_set = set()
+            raise_bad_input_exception('thirdparty recipe requires atleast 1 provider to be passed in sign_in_and_up_feature.providers config')
+        default_providers_set: Set[str] = set()
+        all_providers_set: Set[str] = set()
 
         for provider in providers:
             provider_id = provider.id
@@ -42,8 +47,10 @@ class SignInAndUpFeature:
             is_default = provider.is_default
 
             if not is_default:
-                # if this id is not being used by any other provider, we treat this as the is_default
-                other_providers_with_same_id = list(filter(lambda p: p.id == provider_id and provider != p, providers))
+                # if this id is not being used by any other provider, we treat
+                # this as the is_default
+                other_providers_with_same_id = list(
+                    filter(lambda p: p.id == provider_id and provider != p, providers))
                 if len(other_providers_with_same_id) == 0:
                     # we treat this as the isDefault now.
                     is_default = True
@@ -64,24 +71,39 @@ class SignInAndUpFeature:
         self.providers = providers
 
 
+class InputEmailVerificationConfig:
+    def __init__(self,
+                 get_email_verification_url: Union[Callable[[
+                     User, Dict[str, Any]], Awaitable[str]], None] = None,
+                 create_and_send_custom_email: Union[Callable[[
+                     User, str, Dict[str, Any]], Awaitable[None]], None] = None
+                 ):
+        self.get_email_verification_url = get_email_verification_url
+        self.create_and_send_custom_email = create_and_send_custom_email
+
+
 def email_verification_create_and_send_custom_email(
-        recipe: ThirdPartyRecipe, create_and_send_custom_email):
-    async def func(user, link):
-        user_info = await recipe.recipe_implementation.get_user_by_id(user.id)
+        recipe: ThirdPartyRecipe, create_and_send_custom_email: Callable[[
+            User, str, Dict[str, Any]], Awaitable[None]]) -> Callable[[
+                EmailVerificationUser, str, Dict[str, Any]], Awaitable[None]]:
+    async def func(user: EmailVerificationUser, link: str, user_context: Dict[str, Any]):
+        user_info = await recipe.recipe_implementation.get_user_by_id(user.user_id, user_context)
         if user_info is None:
             raise Exception('Unknown User ID provided')
-        return await create_and_send_custom_email(user_info, link)
+        return await create_and_send_custom_email(user_info, link, user_context)
 
     return func
 
 
 def email_verification_get_email_verification_url(
-        recipe: ThirdPartyRecipe, get_email_verification_url):
-    async def func(user):
-        user_info = await recipe.recipe_implementation.get_user_by_id(user.id)
+        recipe: ThirdPartyRecipe, get_email_verification_url: Callable[[
+            User, Dict[str, Any]], Awaitable[str]]) -> Callable[[
+                EmailVerificationUser, Any], Awaitable[str]]:
+    async def func(user: EmailVerificationUser, user_context: Dict[str, Any]):
+        user_info = await recipe.recipe_implementation.get_user_by_id(user.user_id, user_context)
         if user_info is None:
-            raise Exception(recipe, 'Unknown User ID provided')
-        return await get_email_verification_url(user_info)
+            raise Exception('Unknown User ID provided')
+        return await get_email_verification_url(user_info, user_context)
 
     return func
 
@@ -94,7 +116,8 @@ def validate_and_normalise_email_verification_config(
     if config is None:
         config = InputEmailVerificationConfig()
     if config.create_and_send_custom_email is not None:
-        create_and_send_custom_email = email_verification_create_and_send_custom_email(recipe, config.create_and_send_custom_email)
+        create_and_send_custom_email = email_verification_create_and_send_custom_email(
+            recipe, config.create_and_send_custom_email)
     if config.get_email_verification_url is not None:
         get_email_verification_url = email_verification_get_email_verification_url(recipe,
                                                                                    config.get_email_verification_url)
@@ -109,7 +132,8 @@ def validate_and_normalise_email_verification_config(
 
 class InputOverrideConfig:
     def __init__(self, functions: Union[Callable[[RecipeInterface], RecipeInterface], None] = None,
-                 apis: Union[Callable[[APIInterface], APIInterface], None] = None,
+                 apis: Union[Callable[[APIInterface],
+                                      APIInterface], None] = None,
                  email_verification_feature: Union[EmailVerificationOverrideConfig, None] = None):
         self.functions = functions
         self.apis = apis
@@ -140,11 +164,9 @@ def validate_and_normalise_user_input(
         override: Union[InputOverrideConfig, None] = None) -> ThirdPartyConfig:
     if override is None:
         override = InputOverrideConfig()
-    email_verification_feature = validate_and_normalise_email_verification_config(
-        recipe,
-        email_verification_feature, override)
-    return ThirdPartyConfig(sign_in_and_up_feature, email_verification_feature,
-                            OverrideConfig(functions=override.functions, apis=override.apis)
+    return ThirdPartyConfig(sign_in_and_up_feature, validate_and_normalise_email_verification_config(recipe, email_verification_feature, override),
+                            OverrideConfig(
+                                functions=override.functions, apis=override.apis)
                             )
 
 
@@ -158,30 +180,35 @@ def find_right_provider(
         if provider_id != third_party_id:
             continue
 
-        # first if there is only one provider with third_party_id in the providers array
-        other_providers_with_same_id = list(filter(lambda p: p.id == provider_id and provider != p, providers))
+        # first if there is only one provider with third_party_id in the
+        # providers array
+        other_providers_with_same_id = list(
+            filter(lambda p: p.id == provider_id and provider != p, providers))
         if len(other_providers_with_same_id) == 0:
             # then we always return that.
             return provider
 
-        # otherwise, we look for the is_default provider if client_id is missing
+        # otherwise, we look for the is_default provider if client_id is
+        # missing
         if client_id is None and provider.is_default:
             return provider
 
-        # otherwise, we return a provider that matches based on client Id as well.
+        # otherwise, we return a provider that matches based on client Id as
+        # well.
         if provider.client_id == client_id:
             return provider
 
     return None
 
 
-def verify_id_token_from_jwks_endpoint(id_token: str, jwks_uri: str, audience: str, issuers: List[str]):
+def verify_id_token_from_jwks_endpoint(
+        id_token: str, jwks_uri: str, audience: str, issuers: List[str]) -> Dict[str, Any]:
     jwks_client = PyJWKClient(jwks_uri)
     signing_key = jwks_client.get_signing_key_from_jwt(id_token)
 
-    data = decode(
+    data: Dict[str, Any] = decode(  # type: ignore
         id_token,
-        signing_key.key,
+        signing_key.key,  # type: ignore
         algorithms=["RS256"],
         audience=audience,
         options={"verify_exp": False})
