@@ -11,31 +11,33 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from typing import Union
+from supertokens_python.recipe.emailpassword.types import User
+from typing import Any, Dict
+from supertokens_python.recipe.session import SessionContainer
 import asyncio
 import json
 
 from fastapi import FastAPI
 from fastapi.requests import Request
 from fastapi.testclient import TestClient
-from pytest import fixture
-from pytest import mark
-
-from supertokens_python import init, SupertokensConfig, InputAppInfo
-from supertokens_python.recipe import session, emailpassword
+from pytest import fixture, mark
+from supertokens_python import InputAppInfo, SupertokensConfig, init
 from supertokens_python.framework.fastapi import Middleware
-from supertokens_python.recipe.session.asyncio import create_new_session, refresh_session, get_session
-from tests.utils import (
-    reset, setup_st, clean_st, start_st, sign_up_request
-)
+from supertokens_python.recipe import emailpassword, session
+from supertokens_python.recipe.session.asyncio import (create_new_session,
+                                                       get_session,
+                                                       refresh_session)
+from tests.utils import clean_st, reset, setup_st, sign_up_request, start_st
 
 
-def setup_function(f):
+def setup_function(_):
     reset()
     clean_st()
     setup_st()
 
 
-def teardown_function(f):
+def teardown_function(_):
     reset()
     clean_st()
 
@@ -46,39 +48,43 @@ async def driver_config_client():
     app.add_middleware(Middleware)
 
     @app.get('/login')
-    async def login(request: Request):
+    async def login(request: Request):  # type: ignore
         user_id = 'userId'
         await create_new_session(request, user_id, {}, {})
         return {'userId': user_id}
 
     @app.post('/refresh')
-    async def custom_refresh(request: Request):
+    async def custom_refresh(request: Request):  # type: ignore
         await refresh_session(request)
-        return {}
+        return {}  # type: ignore
 
     @app.get('/info')
-    async def info_get(request: Request):
+    async def info_get(request: Request):  # type: ignore
         await get_session(request, True)
-        return {}
+        return {}  # type: ignore
 
     @app.get('/custom/info')
-    def custom_info(_):
-        return {}
+    def custom_info(_):  # type: ignore
+        return {}  # type: ignore
 
     @app.options('/custom/handle')
-    def custom_handle_options(_):
+    def custom_handle_options(_):  # type: ignore
         return {'method': 'option'}
 
     @app.get('/handle')
-    async def handle_get(request: Request):
-        session = await get_session(request, True)
+    async def handle_get(request: Request):  # type: ignore
+        session: Union[None, SessionContainer] = await get_session(request, True)
+        if session is None:
+            raise Exception("Should never come here")
         return {'s': session.get_handle()}
 
     @app.post('/logout')
-    async def custom_logout(request: Request):
-        session = await get_session(request, True)
+    async def custom_logout(request: Request):  # type: ignore
+        session: Union[None, SessionContainer] = await get_session(request, True)
+        if session is None:
+            raise Exception("Should never come here")
         await session.revoke_session()
-        return {}
+        return {}  # type: ignore
 
     return TestClient(app)
 
@@ -116,10 +122,10 @@ async def test_email_validation_checks_in_generate_token_API(driver_config_clien
 @mark.asyncio
 async def test_that_generated_password_link_is_correct(driver_config_client: TestClient):
     reset_url = None
-    token_info = None
-    rid_info = None
+    token_info: Union[None, str] = None
+    rid_info: Union[None, str] = None
 
-    async def custom_f(user, password_reset_url_with_token):
+    async def custom_f(_: User, password_reset_url_with_token: str, __: Dict[str, Any]):
         nonlocal reset_url, token_info, rid_info
         reset_url = password_reset_url_with_token.split("?")[0]
         token_info = password_reset_url_with_token.split("?")[1].split("&")[0]
@@ -163,8 +169,8 @@ async def test_that_generated_password_link_is_correct(driver_config_client: Tes
 
     assert response_1.status_code == 200
     assert reset_url == "http://supertokens.io/auth/reset-password"
-    assert "token=" in token_info
-    assert "rid=emailpassword" in rid_info
+    assert token_info is not None and "token=" in token_info  # type: ignore pylint: disable=unsupported-membership-test
+    assert rid_info is not None and "rid=emailpassword" in rid_info  # type: ignore pylint: disable=unsupported-membership-test
 
 
 @mark.asyncio
@@ -248,7 +254,7 @@ async def test_token_missing_from_input(driver_config_client: TestClient):
 async def test_valid_token_input_and_passoword_has_changed(driver_config_client: TestClient):
     token_info = None
 
-    async def custom_f(user, password_reset_url_with_token):
+    async def custom_f(_: User, password_reset_url_with_token: str, __: Dict[str, Any]):
         nonlocal token_info
         token_info = password_reset_url_with_token.split("?")[1].split("&")[
             0].split("=")[1]
